@@ -30,7 +30,7 @@ the same terms as the Perl 5 programming language system itself.
 
 package Nagios::NRPE::Client;
 
-our $VERSION = '2.0.3';
+our $VERSION = '2.0.4';
 
 use 5.010_000;
 
@@ -38,8 +38,10 @@ use strict;
 use warnings;
 
 use Carp;
-use IO::Socket;
-use IO::Socket::INET6;
+use IO::Socket ();
+use IO::Socket::INET6 ();
+use IO::Socket::SSL qw(SSL_VERIFY_NONE $SSL_ERROR);
+use Socket qw(SOCK_STREAM AF_INET6 AF_INET);
 use Nagios::NRPE::Utils qw(return_error);
 use Nagios::NRPE::Packet qw(NRPE_PACKET_VERSION_3
   NRPE_PACKET_VERSION_2
@@ -107,6 +109,10 @@ sub new
     $self->{port}     = delete $hash{port}     || 5666;
     $self->{ssl}      = delete $hash{ssl}      || 0;
     $self->{timeout}  = delete $hash{timeout}  || 30;
+    $self->{SSL_cipher_list}  = delete $hash{SSL_cipher_list}  || 'ALL:!MD5:@STRENGTH:@SECLEVEL=0';
+    $self->{SSL_verify_mode}  = delete $hash{SSL_verify_mode}  || SSL_VERIFY_NONE;
+    $self->{SSL_version}      = delete $hash{SSL_version}      || "TLSv1";
+
     bless $self, $class;
 }
 
@@ -149,15 +155,10 @@ sub create_socket
     }
     if ($self->{ssl})
     {
-        eval {
-            # required for new IO::Socket::SSL versions
-            use IO::Socket::SSL;
-        };
 
-        $socket_opts{SSL_cipher_list} = $self->{SSL_cipher_list}
-          || 'ALL:!MD5:@STRENGTH:@SECLEVEL=0';
-        $socket_opts{SSL_verify_mode} = SSL_VERIFY_NONE;
-        $socket_opts{SSL_version}     = 'TLSv1';
+        $socket_opts{SSL_cipher_list} = $self->{SSL_cipher_list};
+        $socket_opts{SSL_verify_mode} = $self->{SSL_verify_mode};
+        $socket_opts{SSL_version} = $self->{SSL_version};
 
         $socket = IO::Socket::SSL->new(%socket_opts);
         if ($SSL_ERROR)
@@ -252,34 +253,48 @@ sub run
     }
     close($socket);
 
-    if (!$response)
+    if($response)
     {
-        $socket = $self->create_socket();
-        if (ref $socket eq "REF")
+        my $responsePacket = $packet->disassemble($response);
+        if ($responsePacket->{packet_version})
         {
-            return ($socket);
+            if ($responsePacket->{packet_version} == 3)
+            {
+                return $responsePacket;
+            }
         }
-        $packet   = Nagios::NRPE::Packet->new();
-        $response = undef;
-        $assembled = $packet->assemble(
-                                       type    => NRPE_PACKET_QUERY,
-                                       check   => $check,
-                                       version => NRPE_PACKET_VERSION_2
-                                      );
-
-        print $socket $assembled;
-        while (<$socket>)
+        else
         {
-            $response .= $_;
-        }
-        close($socket);
-
-        if (!$response)
-        {
-            my $reason = "No output from remote host";
+            my $reason = "Unknown response";
             return return_error($reason);
         }
     }
+
+    $socket = $self->create_socket();
+    if (ref $socket eq "REF")
+    {
+        return ($socket);
+    }
+    $packet   = Nagios::NRPE::Packet->new();
+    $response = undef;
+    $assembled = $packet->assemble(
+                                   type    => NRPE_PACKET_QUERY,
+                                   check   => $check,
+                                   version => NRPE_PACKET_VERSION_2
+                                  );
+
+    print $socket $assembled;
+    while (<$socket>)
+    {
+        $response .= $_;
+    }
+    close($socket);
+
+    if (!$response)
+    {
+        my $reason = "No output from remote host";
+        return return_error($reason);
+    }
     return $packet->disassemble($response);
 }
 
